@@ -243,7 +243,7 @@ int halevt_mount_check_udi(halevt_mount_udi *checked_udi, LibHalContext *hal_ctx
 
   hal_fsusage = libhal_device_get_property_string (hal_ctx, checked_udi->udi, "volume.fsusage", &dbus_error);
   halevt_mount_check_dbus_error (&dbus_error);
-  if (hal_mountpoint == NULL)
+  if (hal_fsusage == NULL)
   {
     fprintf (stderr, "Certainly out of memory, aborting\n");
     exit (1);
@@ -334,7 +334,7 @@ int halevt_mount_check_udi(halevt_mount_udi *checked_udi, LibHalContext *hal_ctx
   return 1;
 }
 
-halevt_mount_remove_udi (halevt_mount_udi *current_udi, 
+void halevt_mount_remove_udi (halevt_mount_udi *current_udi, 
      halevt_mount_udi **previous_udi)
 {
   if ((*previous_udi) == udi_device_mountpoint_root)
@@ -432,7 +432,7 @@ int halevt_mount_print_udis (FILE *file, int print_ignore)
   return 1;
 }
 
-halevt_mount_udi **halevt_mount_find_udi (const char *udi, const char *device, 
+halevt_mount_udi **halevt_mount_find_udi (const char *udi, const char *device,
  const char *mountpoint, const char *device_mountpoint_udi)
 {
   halevt_mount_udi *current_udi = udi_device_mountpoint_root;
@@ -456,7 +456,16 @@ halevt_mount_udi **halevt_mount_find_udi (const char *udi, const char *device,
          (! strcmp (current_udi->udi, device_mountpoint_udi)) || 
          ((current_udi->device != NULL) && (! strcmp(current_udi->device, device_mountpoint_udi))) ||
          ((current_udi->mountpoint != NULL) && (! strcmp(current_udi->mountpoint, device_mountpoint_udi))))))
-    { /* 1 for the new element, 1 for the NULL pointer at the end */
+    { 
+      if ((device != NULL) && (current_udi->device != NULL) && (strcmp (current_udi->device, device)))
+      {
+        fprintf (stderr, "device found %s and device specified %s don't match\n", current_udi->device, device);
+      }
+      if ((mountpoint != NULL) && (current_udi->mountpoint != NULL) && (strcmp (current_udi->mountpoint, mountpoint)))
+      {
+        fprintf (stderr, "mountpoint found %s and mountpoint specified %s don't match\n", current_udi->mountpoint, mountpoint);
+      }
+      /* 1 for the new element, 1 for the NULL pointer at the end */
       if ((list_nr + 2) > list_size)
       {
         list_size = 2*list_size;
@@ -521,12 +530,63 @@ halevt_mount_udi *halevt_mount_new_udi (char *udi)
   return new_udi;
 }
 
-halevt_mount_udi *halevt_mount_add_udi (char *udi, LibHalContext *hal_ctx)
+halevt_mount_udi *halevt_mount_add_udi (char *udi, char *device, LibHalContext *hal_ctx)
 {
-  halevt_mount_udi *new_udi = halevt_mount_new_udi (udi);
+  DBusError dbus_error;
+  char **all_devices;
+  char **current_device;
+  int num_devices;  
+  halevt_mount_udi *new_udi;
+
+  /* 
+    fprintf (stderr, "Add %s, %s\n", udi, device); 
+  */
+  if (device != NULL)
+  {
+    dbus_error_init (&dbus_error);
+    all_devices = libhal_get_all_devices (hal_ctx, &num_devices, &dbus_error);
+    halevt_mount_check_dbus_error (&dbus_error);
+    if (all_devices == NULL)
+    {
+      fprintf (stderr, "HAL didn't return any device\n");
+      exit (1); 
+    }
+
+    current_device = all_devices;
+    while ((*current_device) != NULL)
+    {
+      new_udi = halevt_mount_new_udi (*current_device);
+      if (halevt_mount_check_udi (new_udi, hal_ctx, 0))
+      { /* found a suitable device */
+        if (strcmp (new_udi->device, device))
+        { /* no the same device */
+          free (new_udi);
+        }
+        else
+        {
+          if ((udi != NULL) && (strcmp (udi, new_udi->udi)))
+          {
+            fprintf (stderr, "Device %s is associated with udi %s, but udi %s was specified\n", device, new_udi->udi, udi);
+          }
+          new_udi->next = udi_device_mountpoint_root;
+          udi_device_mountpoint_root = new_udi;
+          return new_udi;
+        }
+      }
+      current_device++;
+    }
+  }
+
+  if (udi == NULL)
+  {
+    fprintf (stderr, "The specified device %s wasn't found\n", device);
+    exit (1);
+  }
+
+  new_udi = halevt_mount_new_udi (udi);
   if (! halevt_mount_check_udi (new_udi, hal_ctx, 1))
   {
-    fprintf (stderr, "The specified udi %s don't seem to exist\n", udi);
+    fprintf (stderr, "The specified udi %s doesn't seem to exist\n", udi);
     exit (1);
   }
   new_udi->next = udi_device_mountpoint_root;
@@ -550,6 +610,10 @@ void halevt_mount_rewrite_device_file (const char *tmp_device_file, const char *
     {
       fprintf (stderr, "Cannot unlink %s: %s\n", tmp_device_file, strerror(errno));      
     }
+    if (unlink (device_file) < 0)
+    {
+      fprintf (stderr, "Cannot unlink %s: %s\n", device_file, strerror(errno));      
+    }
   }
   else 
   {
@@ -565,20 +629,20 @@ void usage()
   printf (
 " Usage: halevt-mount [option]\n\
         halevt-umount [option]\n\
- Mount or umount device through hal and keep a track of devices.\n\
+ Mount or umount device through hal and keep a track of devices handled.\n\
  When called as halevt-umount the default is to unmount, when called as\n\
  halevt-mount the default is to mount.\n\
  Option -c, -l, -r, -s or -w change the operation performed.\n\
-   -a               when removing or unmounting, do this for all the \n\
-                    known devices. When listing, list all the existing\n\
-                    devices\n\
-   -c               remove track of handled device\n\
+   -a               When listing, list all the existing devices.\n\
+                    For the other operations, perform the operation on\n\
+                    all the handled devices\n\
+   -c               remove the handled device\n\
    -d x             use device <x>\n\
-   -f x             use file <x> to store information about the known\n\
+   -f x             use file <x> to store information about the handled\n\
                     devices instead of the default\n\
    -i x             use lock file and device information file in the \n\
                     directory <x> (if they are not already specified)\n\
-   -l               list known devices. Formatted with, on each line:\n\
+   -l               list handled devices. Formatted with, on each line:\n\
                                  udi:device:mount point\n\
    -p x             use mountpoint <x>\n\
    -m x             set mount umask to <x> if run as a system user\n\
@@ -588,12 +652,12 @@ void usage()
    -s               sync the information about the handled devices\n\
                     with the informations known by HAL\n\
    -u x             use udi <x>\n\
-   -w               add device to the handled devices list\n\n\
- Additional arguments are considered to be mount points, devices or udi\n\
- to unmount.\n\n\
+   -w               add to the handled devices\n\n\
+ Additional argument is considered to be a mount point, a device or an udi\n\
+ when unmounting or removing.\n\n\
  If the commands are not called with -s after changes in the state of the\n\
  devices (after mounting, unmounting, removing devices) , the information\n\
- available for halevt-mount and halevt-umount may become out of ssync with\n\
+ available for halevt-mount and halevt-umount may become out of sync with\n\
  the state of the system. \n\
  "
 ); 
@@ -784,6 +848,7 @@ int main (int argc, char **argv)
       cmd = "mount";
     }
   }
+  /* fprintf (stderr, "Executing cmd: %s\n", cmd); */
 
   if (optind < argc) { arg_index = optind; }
   
@@ -886,12 +951,12 @@ int main (int argc, char **argv)
     }
     goto end;
   }
-  
   /* initialize hal, we are going to synchronize the informations with the
    hal informations */
   if ((hal_ctx = libhal_ctx_new()) == NULL)
   {
     fprintf (stderr, "Failed to create HAL context\n");
+    exit(1);
   }
   dbus_error_init (&dbus_error);
   dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &dbus_error);
@@ -920,7 +985,7 @@ int main (int argc, char **argv)
     fprintf (stdout, "Devices handled:\n");
     halevt_mount_print_udis (stdout, 0);
 
-    /* get and add all the devices, reusing the same udi root list */
+    /* get and add all the devices */
     all_devices = libhal_get_all_devices (hal_ctx, &num_devices, &dbus_error);
     halevt_mount_check_dbus_error (&dbus_error);
     if (all_devices == NULL)
@@ -928,6 +993,7 @@ int main (int argc, char **argv)
       fprintf (stderr, "HAL didn't return any device\n");
       exit (1); 
     }
+    /* reuse the same udi root list */
     udi_device_mountpoint_root = NULL;
     current_device = all_devices;
     while ((*current_device) != NULL)
@@ -945,13 +1011,13 @@ int main (int argc, char **argv)
   }
   else if ((! strcmp (cmd, "mount")) || (! strcmp (cmd, "add")))
   {
-    if (udi == NULL)
+    if ((udi == NULL) && (device == NULL))
     {
-      fprintf (stderr, "An udi is needed\n");
+      fprintf (stderr, "An udi or a device is needed\n");
       exit (1);
     }
-    udi_found = halevt_mount_find_udi (udi, NULL, NULL, NULL);
-    if (udi_found[0] == NULL) { halevt_mount_add_udi (udi, hal_ctx); }
+    udi_found = halevt_mount_find_udi (udi, device, NULL, NULL);
+    if (udi_found[0] == NULL) { halevt_mount_add_udi (udi, device, hal_ctx); }
     /* write the new udi structure to the temporary file */
     if ((out_fd = fopen (tmp_device_file, "w")) == NULL)
     {
@@ -1023,12 +1089,6 @@ int main (int argc, char **argv)
     }
     else 
     {
-      if ((udi != NULL) && (! strcmp (cmd, "umount")))
-      { /* when umounting, an udi is given, and it is not found try to add it */
-        udi_found = halevt_mount_find_udi (udi, NULL, NULL, NULL);
-        if (udi_found[0] == NULL) { halevt_mount_add_udi (udi, hal_ctx); }
-      }
-
       if (arg_index) { searched_device_mountpoint = argv[arg_index]; }
 
       udi_found = halevt_mount_find_udi (udi, device, mountpoint, searched_device_mountpoint);
